aboutsummaryrefslogtreecommitdiff
path: root/src/cmd/internal
diff options
context:
space:
mode:
Diffstat (limited to 'src/cmd/internal')
-rw-r--r--src/cmd/internal/hash/hash.go41
1 files changed, 29 insertions, 12 deletions
diff --git a/src/cmd/internal/hash/hash.go b/src/cmd/internal/hash/hash.go
index 20edc72c20..a37368f50e 100644
--- a/src/cmd/internal/hash/hash.go
+++ b/src/cmd/internal/hash/hash.go
@@ -5,22 +5,33 @@
// Package hash implements hash functions used in the compiler toolchain.
package hash
+// TODO(rsc): Delete the 16 and 20 forms and use 32 at all call sites.
+
import (
- "crypto/md5"
- "crypto/sha1"
"crypto/sha256"
"hash"
)
const (
- // Size32 is the size of 32 bytes hash checksum.
- Size32 = sha256.Size
- // Size20 is the size of 20 bytes hash checksum.
- Size20 = sha1.Size
- // Size16 is the size of 16 bytes hash checksum.
- Size16 = md5.Size
+ // Size32 is the size of the 32-byte hash checksum.
+ Size32 = 32
+ // Size20 is the size of the 20-byte hash checksum.
+ Size20 = 20
+ // Size16 is the size of the 16-byte hash checksum.
+ Size16 = 16
)
+type shortHash struct {
+ hash.Hash
+ n int
+}
+
+func (h *shortHash) Sum(b []byte) []byte {
+ old := b
+ sum := h.Hash.Sum(b)
+ return sum[:len(old)+h.n]
+}
+
// New32 returns a new [hash.Hash] computing the 32 bytes hash checksum.
func New32() hash.Hash {
h := sha256.New()
@@ -30,12 +41,12 @@ func New32() hash.Hash {
// New20 returns a new [hash.Hash] computing the 20 bytes hash checksum.
func New20() hash.Hash {
- return sha1.New()
+ return &shortHash{New32(), 20}
}
// New16 returns a new [hash.Hash] computing the 16 bytes hash checksum.
func New16() hash.Hash {
- return md5.New()
+ return &shortHash{New32(), 16}
}
// Sum32 returns the 32 bytes checksum of the data.
@@ -47,10 +58,16 @@ func Sum32(data []byte) [Size32]byte {
// Sum20 returns the 20 bytes checksum of the data.
func Sum20(data []byte) [Size20]byte {
- return sha1.Sum(data)
+ sum := Sum32(data)
+ var short [Size20]byte
+ copy(short[:], sum[4:])
+ return short
}
// Sum16 returns the 16 bytes checksum of the data.
func Sum16(data []byte) [Size16]byte {
- return md5.Sum(data)
+ sum := Sum32(data)
+ var short [Size16]byte
+ copy(short[:], sum[8:])
+ return short
}